@@ -292,7 +292,7 @@ class BaseParser(object):
                      'dollar_lbrace', 'dollar_lbracket', 'try',
                      'bang_lparen', 'bang_lbracket', 'comma', 'rparen',
                      'rbracket', 'at_lparen', 'atdollar_lparen', 'indent',
-                     'dedent', 'newline']
+                     'dedent', 'newline', 'lambda']
         for rule in tok_rules:
             self._tok_rule(rule)
 
@@ -646,21 +646,37 @@ class BaseParser(object):
         p[0] = ast.arguments(args=[], vararg=None, kwonlyargs=[],
                              kw_defaults=[], kwarg=p[2], defaults=[])
 
-    def p_typedargslist_times4(self, p):
-        """typedargslist : TIMES tfpdef_opt comma_pow_tfpdef_opt"""
+    def p_typedargslist_times4_tfpdef(self, p):
+        """typedargslist : TIMES tfpdef comma_pow_tfpdef_opt"""
+        # *args, **kwargs
         p0 = ast.arguments(args=[], vararg=None, kwonlyargs=[], kw_defaults=[],
                            kwarg=p[3], defaults=[])
         self._set_var_args(p0, p[2], None)
         p[0] = p0
 
-    def p_typedargslist_times5(self, p):
-        """typedargslist : TIMES tfpdef_opt comma_tfpdef_list comma_pow_tfpdef_opt"""
+    def p_typedargslist_times4_comma(self, p):
+        """typedargslist : TIMES comma_pow_tfpdef"""
+        # *, **kwargs
+        p0 = ast.arguments(args=[], vararg=None, kwonlyargs=[], kw_defaults=[],
+                           kwarg=p[2], defaults=[])
+        p[0] = p0
+
+    def p_typedargslist_times5_tdpdef(self, p):
+        """typedargslist : TIMES tfpdef comma_tfpdef_list comma_pow_tfpdef_opt"""
         # *args, x, **kwargs
         p0 = ast.arguments(args=[], vararg=None, kwonlyargs=[], kw_defaults=[],
                            kwarg=p[4], defaults=[])
         self._set_var_args(p0, p[2], p[3])  # *args
         p[0] = p0
 
+    def p_typedargslist_times5_comma(self, p):
+        """typedargslist : TIMES comma_tfpdef_list comma_pow_tfpdef_opt"""
+        # *, x, **kwargs
+        p0 = ast.arguments(args=[], vararg=None, kwonlyargs=[], kw_defaults=[],
+                           kwarg=p[3], defaults=[])
+        self._set_var_args(p0, None, p[2])  # *args
+        p[0] = p0
+
     def p_typedargslist_t5(self, p):
         """typedargslist : tfpdef equals_test_opt comma_tfpdef_list_opt comma_opt"""
         # x
@@ -733,6 +749,9 @@ class BaseParser(object):
     def _set_args_def(self, argmts, vals, kwargs=False):
         args, defs = (argmts.kwonlyargs, argmts.kw_defaults) if kwargs else \
                      (argmts.args, argmts.defaults)
+        if vals is None and kwargs:
+            loc = self.currloc(self.lineno, self.col)
+            self._parse_error('named arguments must follow bare *', loc)
         for v in vals:
             args.append(v['arg'])
             d = v['default']
@@ -758,7 +777,7 @@ class BaseParser(object):
             self._set_args_def(p0, p3)
 
     def _set_var_args(self, p0, vararg, kwargs):
-        if vararg is None:
+        if vararg is None and kwargs is not None:
             self._set_args_def(p0, kwargs, kwargs=True)
         elif vararg is not None and kwargs is None:
             # *args
@@ -1503,15 +1522,15 @@ class BaseParser(object):
         p[0] = p[1]
 
     def p_lambdef(self, p):
-        """lambdef : LAMBDA varargslist_opt COLON test"""
-        p2, p4 = p[2], p[4]
+        """lambdef : lambda_tok varargslist_opt COLON test"""
+        p1, p2, p4 = p[1], p[2], p[4]
         if p2 is None:
             args = ast.arguments(args=[], vararg=None, kwonlyargs=[],
                                  kw_defaults=[], kwarg=None, defaults=[])
         else:
             args = p2
-        p0 = ast.Lambda(args=args, body=p4, lineno=self.lineno,
-                        col_offset=self.col)
+        p0 = ast.Lambda(args=args, body=p4, lineno=p1.lineno,
+                        col_offset=p1.lexpos)
         p[0] = p0
 
     def p_lambdef_nocond(self, p):
